@@ -71,6 +71,8 @@ pub struct Cache<A: BackendApi, S: Storage, Q: Querier> {
     type_api: PhantomData<A>,
     type_storage: PhantomData<S>,
     type_querier: PhantomData<Q>,
+    /// To prevent concurrent access to `WasmerInstance::new`
+    instantiation_lock: Mutex<()>,
 }
 
 #[derive(PartialEq, Debug)]
@@ -131,6 +133,7 @@ where
             type_storage: PhantomData::<S>,
             type_api: PhantomData::<A>,
             type_querier: PhantomData::<Q>,
+            instantiation_lock: Mutex::new(()),
         })
     }
 
@@ -151,7 +154,7 @@ where
 
     pub fn save_wasm(&self, wasm: &[u8]) -> VmResult<Checksum> {
         check_wasm(wasm, &self.supported_features)?;
-        let module = compile(wasm, None)?;
+        let module = compile(wasm, None, &[])?;
 
         let mut cache = self.inner.lock().unwrap();
         let checksum = save_wasm_to_disk(&cache.wasm_path, wasm)?;
@@ -224,7 +227,7 @@ where
 
         // Re-compile from original Wasm bytecode
         let code = self.load_wasm_with_path(&cache.wasm_path, checksum)?;
-        let module = compile(&code, Some(cache.instance_memory_limit))?;
+        let module = compile(&code, Some(cache.instance_memory_limit), &[])?;
         // Store into the fs cache too
         cache.fs_cache.store(checksum, &module)?;
         let module_size = loupe::size_of_val(&module);
@@ -246,43 +249,52 @@ where
     }
 
     /// Returns an Instance tied to a previously saved Wasm.
-    /// Depending on availability, this is either generated from a cached instance, a cached module or Wasm code.
+    ///
+    /// It takes a module from cache or Wasm code and instantiates it.
     pub fn get_instance(
         &self,
         checksum: &Checksum,
         backend: Backend<A, S, Q>,
         options: InstanceOptions,
     ) -> VmResult<Instance<A, S, Q>> {
+        let module = self.get_module(checksum)?;
+        let instance = Instance::from_module(
+            &module,
+            backend,
+            options.gas_limit,
+            options.print_debug,
+            None,
+            Some(&self.instantiation_lock),
+        )?;
+        Ok(instance)
+    }
+
+    /// Returns a module tied to a previously saved Wasm.
+    /// Depending on availability, this is either generated from a memory cache, file system cache or Wasm code.
+    /// This is part of `get_instance` but pulled out to reduce the locking time.
+    fn get_module(&self, checksum: &Checksum) -> VmResult<wasmer::Module> {
         let mut cache = self.inner.lock().unwrap();
         // Try to get module from the pinned memory cache
         if let Some(module) = cache.pinned_memory_cache.load(checksum)? {
             cache.stats.hits_pinned_memory_cache += 1;
-            let instance =
-                Instance::from_module(&module, backend, options.gas_limit, options.print_debug)?;
-            return Ok(instance);
+            return Ok(module);
         }
 
         // Get module from memory cache
         if let Some(module) = cache.memory_cache.load(checksum)? {
             cache.stats.hits_memory_cache += 1;
-            let instance = Instance::from_module(
-                &module.module,
-                backend,
-                options.gas_limit,
-                options.print_debug,
-            )?;
-            return Ok(instance);
+            return Ok(module.module);
         }
 
         // Get module from file system cache
         let store = make_runtime_store(Some(cache.instance_memory_limit));
         if let Some(module) = cache.fs_cache.load(checksum, &store)? {
             cache.stats.hits_fs_cache += 1;
-            let instance =
-                Instance::from_module(&module, backend, options.gas_limit, options.print_debug)?;
             let module_size = loupe::size_of_val(&module);
-            cache.memory_cache.store(checksum, module, module_size)?;
-            return Ok(instance);
+            cache
+                .memory_cache
+                .store(checksum, module.clone(), module_size)?;
+            return Ok(module);
         }
 
         // Re-compile module from wasm
@@ -292,13 +304,13 @@ where
         // stored the old module format.
         let wasm = self.load_wasm_with_path(&cache.wasm_path, checksum)?;
         cache.stats.misses += 1;
-        let module = compile(&wasm, Some(cache.instance_memory_limit))?;
-        let instance =
-            Instance::from_module(&module, backend, options.gas_limit, options.print_debug)?;
+        let module = compile(&wasm, Some(cache.instance_memory_limit), &[])?;
         cache.fs_cache.store(checksum, &module)?;
         let module_size = loupe::size_of_val(&module);
-        cache.memory_cache.store(checksum, module, module_size)?;
-        Ok(instance)
+        cache
+            .memory_cache
+            .store(checksum, module.clone(), module_size)?;
+        Ok(module)
     }
 }
 
@@ -363,10 +375,9 @@ mod tests {
     use cosmwasm_std::{coins, Empty};
     use std::fs::OpenOptions;
     use std::io::Write;
-    use std::iter::FromIterator;
     use tempfile::TempDir;
 
-    const TESTING_GAS_LIMIT: u64 = 4_000_000;
+    const TESTING_GAS_LIMIT: u64 = 500_000_000_000; // ~0.5ms
     const TESTING_MEMORY_LIMIT: Size = Size::mebi(16);
     const TESTING_OPTIONS: InstanceOptions = InstanceOptions {
         gas_limit: TESTING_GAS_LIMIT,
@@ -378,7 +389,7 @@ mod tests {
     static IBC_CONTRACT: &[u8] = include_bytes!("../testdata/ibc_reflect.wasm");
 
     fn default_features() -> HashSet<String> {
-        features_from_csv("staking")
+        features_from_csv("iterator,staking")
     }
 
     fn make_testing_options() -> CacheOptions {
@@ -391,9 +402,11 @@ mod tests {
     }
 
     fn make_stargate_testing_options() -> CacheOptions {
+        let mut feature = default_features();
+        feature.insert("stargate".into());
         CacheOptions {
             base_dir: TempDir::new().unwrap().into_path(),
-            supported_features: features_from_csv("staking,stargate"),
+            supported_features: feature,
             memory_cache_size: TESTING_MEMORY_CACHE_SIZE,
             instance_memory_limit: TESTING_MEMORY_LIMIT,
         }
@@ -984,6 +997,7 @@ mod tests {
             AnalysisReport {
                 has_ibc_entry_points: true,
                 required_features: HashSet::from_iter(vec![
+                    "iterator".to_string(),
                     "staking".to_string(),
                     "stargate".to_string()
                 ]),
