@@ -10,6 +10,7 @@ use crate::static_analysis::{deserialize_wasm, ExportInfo};
 /// Lists all imports we provide upon instantiating the instance in Instance::from_module()
 /// This should be updated when new imports are added
 const SUPPORTED_IMPORTS: &[&str] = &[
+    "env.abort",
     "env.db_read",
     "env.db_write",
     "env.db_remove",
@@ -41,9 +42,11 @@ const REQUIRED_EXPORTS: &[&str] = &[
 ];
 
 const INTERFACE_VERSION_PREFIX: &str = "interface_version_";
-/// Only one version is supported right now. This could potentially turn into a list
-/// later, but maybe this never happens and new functionality is only added via features.
-const SUPPORTED_INTERFACE_VERSION: &str = "interface_version_7";
+const SUPPORTED_INTERFACE_VERSIONS: &[&str] = &[
+    "interface_version_8",
+    #[cfg(feature = "allow_interface_version_7")]
+    "interface_version_7",
+];
 
 const MEMORY_LIMIT: u32 = 512; // in pages
 
@@ -105,21 +108,16 @@ fn check_interface_version(module: &Module) -> VmResult<()> {
             ))
         } else {
             // Exactly one interface version found
-
-            match first_interface_version_export.as_str() {
-                // Ok
-                SUPPORTED_INTERFACE_VERSION => Ok(()),
-                // Well known old versions for better error messages
-                "interface_version_6" => Err(VmError::static_validation_err(
-                    "Wasm contract has incompatible CosmWasm 0.15 marker export interface_version_6 (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
-                )),
-                "interface_version_5" => Err(VmError::static_validation_err(
-                    "Wasm contract has incompatible CosmWasm 0.14 marker export interface_version_5 (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
-                )),
-                // Unknown version
-                _ => Err(VmError::static_validation_err(
-                    "Wasm contract has unknown interface_version_* marker export (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)",
-                )),
+            let version_str = first_interface_version_export.as_str();
+            if SUPPORTED_INTERFACE_VERSIONS
+                .iter()
+                .any(|&v| v == version_str)
+            {
+                Ok(())
+            } else {
+                Err(VmError::static_validation_err(
+                        "Wasm contract has unknown interface_version_* marker export (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)",
+                ))
             }
         }
     } else {
@@ -215,7 +213,7 @@ mod tests {
         match check_wasm(CONTRACT_0_15, &default_features()) {
             Err(VmError::StaticValidationErr { msg, .. }) => assert_eq!(
                 msg,
-                "Wasm contract has incompatible CosmWasm 0.15 marker export interface_version_6 (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
+                "Wasm contract has unknown interface_version_* marker export (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
             ),
             Err(e) => panic!("Unexpected error {:?}", e),
             Ok(_) => panic!("This must not succeeed"),
@@ -224,7 +222,7 @@ mod tests {
         match check_wasm(CONTRACT_0_14, &default_features()) {
             Err(VmError::StaticValidationErr { msg, .. }) => assert_eq!(
                 msg,
-                "Wasm contract has incompatible CosmWasm 0.14 marker export interface_version_5 (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
+                "Wasm contract has unknown interface_version_* marker export (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)"
             ),
             Err(e) => panic!("Unexpected error {:?}", e),
             Ok(_) => panic!("This must not succeeed"),
@@ -348,7 +346,7 @@ mod tests {
                 (func (type 0) nop)
                 (export "add_one" (func 0))
                 (export "allocate" (func 0))
-                (export "interface_version_7" (func 0))
+                (export "interface_version_8" (func 0))
                 (export "deallocate" (func 0))
                 (export "instantiate" (func 0))
             )"#,
@@ -357,6 +355,25 @@ mod tests {
         let module = deserialize_wasm(&wasm).unwrap();
         check_interface_version(&module).unwrap();
 
+        #[cfg(feature = "allow_interface_version_7")]
+        {
+            // valid legacy version
+            let wasm = wat::parse_str(
+                r#"(module
+                    (type (func))
+                    (func (type 0) nop)
+                    (export "add_one" (func 0))
+                    (export "allocate" (func 0))
+                    (export "interface_version_7" (func 0))
+                    (export "deallocate" (func 0))
+                    (export "instantiate" (func 0))
+                )"#,
+            )
+            .unwrap();
+            let module = deserialize_wasm(&wasm).unwrap();
+            check_interface_version(&module).unwrap();
+        }
+
         // missing
         let wasm = wat::parse_str(
             r#"(module
@@ -387,8 +404,8 @@ mod tests {
                 (func (type 0) nop)
                 (export "add_one" (func 0))
                 (export "allocate" (func 0))
-                (export "interface_version_7" (func 0))
                 (export "interface_version_8" (func 0))
+                (export "interface_version_9" (func 0))
                 (export "deallocate" (func 0))
                 (export "instantiate" (func 0))
             )"#,
@@ -421,7 +438,7 @@ mod tests {
         let module = deserialize_wasm(&wasm).unwrap();
         match check_interface_version(&module).unwrap_err() {
             VmError::StaticValidationErr { msg, .. } => {
-                assert_eq!(msg, "Wasm contract has incompatible CosmWasm 0.15 marker export interface_version_6 (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)");
+                assert_eq!(msg, "Wasm contract has unknown interface_version_* marker export (see https://github.com/CosmWasm/cosmwasm/blob/main/packages/vm/README.md)");
             }
             err => panic!("Unexpected error {:?}", err),
         }
