@@ -1,7 +1,6 @@
 // The external interface is `use cosmwasm_vm::testing::X` for all integration testing symbols, no matter where they live internally.
 
 mod calls;
-mod ibc_calls;
 mod instance;
 mod mock;
 mod querier;
@@ -9,7 +8,7 @@ mod storage;
 
 pub use calls::{execute, instantiate, migrate, query, reply, sudo};
 #[cfg(feature = "stargate")]
-pub use ibc_calls::{
+pub use calls::{
     ibc_channel_close, ibc_channel_connect, ibc_channel_open, ibc_packet_ack, ibc_packet_receive,
     ibc_packet_timeout,
 };
